--[[-------------------------------------------------------------------------
	FixModSounds
	Copyright (C) 2010-2011  Morsker
	Please contact Morsker through PM on wowace.com.

	Corrects unspecified mod sounds to use the Master channel.

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program. If not, see <http://www.gnu.org/licenses/>.
---------------------------------------------------------------------------]]
local addonName, addonScope = ...
local FixModSounds = CreateFrame("Frame", "FixModSounds")
local L = addonScope.L
local db

----------------
--  Upvalues  --
----------------

-- GLOBALS: _G
-- GLOBALS: FixModSoundsDB
-- GLOBALS: table
-- GLOBALS: pairs
-- GLOBALS: sort
-- GLOBALS: NONE
-- GLOBALS: UNKNOWN
local debugstack = debugstack
local strmatch = strmatch

--------------
--  Locals  --
--------------

local DBVERSION = 2
local real_PlaySoundFile = PlaySoundFile
local trace = false
local ignoredSounds
local ignoredMods

-------------
--  Utils  --
-------------

-- Print with colored prefix.
local print
do
	local printPrefix = "|cff33ff99".."FixModSounds".."|r:"
	function print(...)
		return _G.print(printPrefix, ...)
	end
end

local function SortedKeys(hashTable)
	local t = {}
	for k,_ in pairs(hashTable) do
		t[#t+1] = k
	end
	sort(t)
	return t
end

---------------
--  Members  --
---------------

function FixModSounds.hooked_PlaySoundFile(soundFile, soundChannel)
	if soundChannel ~= nil then
		-- The mod only interacts with sounds that specify no channel.
		-- Others get trivially delegated.
		if trace then
			local callingMod = strmatch(debugstack(2, 1, 0), [[\AddOns\(.-)\]]) or UNKNOWN
			local str = L["Skipping sound ($s) from mod ($m) on channel ($c)"]
			str = str:gsub("$s", soundFile)
			str = str:gsub("$m", callingMod)
			str = str:gsub("$c", soundChannel)
			print(str)
		end
		real_PlaySoundFile(soundFile, soundChannel)
		return
	end

	local callingMod = strmatch(debugstack(2, 1, 0), [[\AddOns\(.-)\]]) or UNKNOWN
	if ignoredSounds[soundFile] or ignoredMods[callingMod] then
		if trace then
			local str = L["Ignoring sound ($s) from mod ($m)"]
			str = str:gsub("$s", soundFile)
			str = str:gsub("$m", callingMod)
			print(str)
		end
		return real_PlaySoundFile(soundFile)
	else
		if trace then
			local str = L["Turning up sound ($s) from mod ($m)"]
			str = str:gsub("$s", soundFile)
			str = str:gsub("$m", callingMod)
			print(str)
		end
		return real_PlaySoundFile(soundFile, "Master")
	end
end

-------------------
--  CommandLine  --
-------------------

function FixModSounds.HandleChatCommand(msg)
	local cmd, arg = msg:match("^([^%s]+)(.*)$")
	cmd = cmd and cmd:lower() or ""
	arg = arg and arg:trim() or ""
	if cmd == "trace" then
		trace = not trace
		print(trace and L["Trace On."] or L["Trace Off."])
	elseif cmd == "ignoremod" then
		if arg == "" then
			local t = { L["Ignored Mods:"] }
			local mods = SortedKeys(ignoredMods)
			if #mods == 0 then
				mods[1] = "<"..NONE..">"
			end
			for i = 1,#mods do
				t[#t+1] = "\n  "
				t[#t+1] = mods[i]
			end
			print(table.concat(t))
		else
			local str
			if ignoredMods[arg] then
				str, ignoredMods[arg] = L["Unignored mod '$m'"], nil
			else
				str, ignoredMods[arg] = L["Ignored mod '$m'"], true
			end
			str = str:gsub("$m", arg)
			print(str)
		end
	elseif cmd == "ignoresound" then
		if arg == "" then
			local t = { L["Ignored Sounds:"] }
			local sounds = SortedKeys(ignoredSounds)
			if #sounds == 0 then
				sounds[1] = "<"..NONE..">"
			end
			for i = 1,#sounds do
				t[#t+1] = "\n  "
				t[#t+1] = sounds[i]
			end
			print(table.concat(t))
		else
			local str
			if ignoredSounds[arg] then
				str, ignoredSounds[arg] = L["Unignored sound '$s'"], nil
			else
				str, ignoredSounds[arg] = L["Ignored sound '$s'"], true
			end
			str = str:gsub("$s", arg)
			print(str)
		end
	else
		print("usage:"
			.. "\n  /fixsound trace "..L["- Toggle tracing, letting you see which mods play which sounds."]
			.. "\n  /fixsound ignoremod "..L["<mod> - Don't turn up volume for this mod's sound."]
			.. "\n  /fixsound ignoresound "..L["<sound> - Don't turn up volume for this sound."]
			)
	end
end

-- GLOBALS: SLASH_FIXMODSOUNDS_FIXMODSOUNDS1
-- GLOBALS: SLASH_FIXMODSOUNDS_FIXMODSOUNDS2
-- GLOBALS: SLASH_FIXMODSOUNDS_FIXMODSOUNDS3
SLASH_FIXMODSOUNDS_FIXMODSOUNDS1 = "/fixmodsounds"
SLASH_FIXMODSOUNDS_FIXMODSOUNDS2 = "/fixsounds"
SLASH_FIXMODSOUNDS_FIXMODSOUNDS3 = "/fixsound"
SlashCmdList.FIXMODSOUNDS_FIXMODSOUNDS = FixModSounds.HandleChatCommand

-----------------------
--  Script Handlers  --
-----------------------

function FixModSounds:OnEvent(event, ...)
	if event == "ADDON_LOADED" then
		local name = ...
		if name == addonName then
			self:UnregisterEvent("ADDON_LOADED")

			-- db
			db = FixModSoundsDB or {
				dbversion = DBVERSION,
				ignoredSounds = {
					["Interface\\Quiet.ogg"] = true, -- noop sound used by LibSharedMedia
				},
				ignoredMods = {
				},
			}
			if db.dbversion < DBVERSION then
				if db.version < 2 then
					-- Older versions of LSM used a different sentinel value for this.
					if db.ignoredSounds["Interface\\Quiet.mp3"] then
						db.ignoredSounds["Interface\\Quiet.mp3"] = nil
						db.ignoredSounds["Interface\\Quiet.ogg"] = true
					end
				end
				db.dbversion = DBVERSION
			end
			self.db = db
			ignoredSounds = db.ignoredSounds
			ignoredMods = db.ignoredMods

			-- hook
			self.real_PlaySoundFile = real_PlaySoundFile
			_G.PlaySoundFile = self.hooked_PlaySoundFile
		end
	end
end

-------------
--  Setup  --
-------------

FixModSounds:SetScript("OnEvent", FixModSounds.OnEvent)
FixModSounds:RegisterEvent("ADDON_LOADED")
